/**
 *	tbl.h	n.k.	june 1992
 *	tb2.h	k.f.	july 1996
 * 
 * v1.0 : june9, 1992 n.k. 
 * v1.1 : june16, 1992 n.k. add TBL_VER, TBL_WARN
 * v2.0 : july29, 1996 k.f. 
 * v2.1 : feb14, 1997 k.f. 
 *
 * #define const			I should consider this again
 *
 * Modifications:
 *  dec.26.1997. k.f change default directory into /control/tbl/ on unix
 *  sep.2.1999.  k.f use "struct **xxx" instead of "struct (*xxx)[]"
 *
 * @(#)tb2.h, v2.1.5, n.k, k.f, linac table definition, jun.1992-sep.1999
 *
 *****

  standard format of tables

    !separator ' '
    !keyword   "NAME NODE KEY1 KEY2 KEY3"
    !format    "%s %s %d %d %x %f"
    name1  node1  10  11  a0  1.2
    name2  node2  20  21  bc  3.4

  current feature
    no limit for number/length of keywords/names (in version 2)
    no limit for number of string parameters (in version 2)
    types of parameters are string, int, long, float, double (in version 2)
    many tables can be handled (in version 2)
    default directory of the table file can be specified in TBLDIR
    format should be used for printf as well as scanf for tb2key to work
    first keyword should be 'NAME'

**/

#ifndef _TB2_H
#define _TB2_H

#include <stdio.h>		/* for dynhash.h and debug */
#ifdef TB2INTERNAL
#include "dynhash.h"		/* for tb2_hash.c */
#endif /* TB2INTERNAL */

/*** tbl definitions ***/
#define TBL_VER "v2.1"		/* version v2.0 */

/* debug symbols */
#ifdef DEBUG2
#undef DEBUG1
#define DEBUG1
#define DEBUG
#endif /* DEBUG2 */

#ifdef _OSK			/* for ultra-C */
#define OSK			/* for microware-C */
#endif /* _OSK */

#ifdef TBL_WARN
      /* =0 show no warning message, for libtb2.a */
      /* =1 show warning message to stdout, for librtb2.a */
      /* =2 show warning message to stderr, for libtb2.a */
#if (TBL_WARN == 0)
#undef TBL_WARN
#elif (TBL_WARN != 2)
#define WARN_DEST stdout
#else  /* TBL_WARN */
#define WARN_DEST stderr
#endif  /* TBL_WARN */
#endif /* TBL_WARN */

/*** tbl definitions ***/
#ifdef ebcdic			/* for mx3000/pfumx */
#define DEF_DIR "/usr/user/control/tbl/"
#endif
#ifdef OSK			/* os9 network environment */
#define DEF_DIR "/td/USR/TBL/"
#endif
#ifdef MSDOS
#define DEF_DIR "D:\\TBL\\"	/* msdos ram disk */
#endif
#ifndef DEF_DIR			/* default table directory for others */
#define DEF_DIR "/control/tbl/"
#endif
                          /* default directory for the table file */
#define N_KEYS 128		/* maximum # (buff size) for keyword entry */
#define ORIG_N_KEYS 16		/* in old version */
#define L_KEY 64		/* size for each keyword (char *) */
#define L_FORMAT 1024		/* buff size for format entry */
#define L_FNAME 132		/* buff size for filename */
#define L_BUFFSIZE 1024		/* buff size for file i/o buffer */
#define DEF_SEPARATOR  '\t'	/* default separator is TAB */
#define DEF_FORMAT "%s"		/* default format */
#define DEF_NKEYS  1		/* default # of keywords */
/* #define DEF_KEY1 "NAME" */	/* default keyword (1st key) */
/* #define N_BUFMAX 512 */      /* buffer size for data strage */
#define L_NAME 64		/* size for each NAME  entry */
#define L_NODE 64		/* size for each NODE  entry */
#define N_POS (N_KEYS)		/* buffer size for positon() */

/* #define TBL_FIRST_SERIAL 1 *//* first serial number,0 in v1,v2.1, 1 in v2 */
/* #define TBL_FIRST_SERIAL 0 *//* first serial number,0 in v1,v2.1, 1 in v2 */
#define TBL_FIRST_SERIAL 1	/* first serial number,0 in v1,v2.1, 1 in v2 */

/*** definitions for error codes ***/

  /* [1] com="READ", tb2_read() */
#define TBL_E_NOTFOUND    (-71)  /* table not found or unable to open */
#define TBL_E_ILLLINE     (-72)  /* table contains illegal formatted line */
#define TBL_E_NULLTBL     (-73)  /* table contains no data */
#define TBL_E_BUFFSIZE    (-74)  /* table contains too many data */
#define TBL_E_KEYWORDS    (-75)  /* keyword specified is illegal */

  /* [2] _tb2_num() */
#define TBL_E_UNREAD      (-76)  /* table file does not read yet */
#define TBL_E_NAME        (-12)  /* specified name not found in the table */

  /* [3] tb2_int() */
/* errors defined tb2_int() */
#define TBL_E_KEY         (-77)  /* keyword not found, or illegal keyword */

  /* [4] tb2_node() */
/* errors defined tb2_node() */
#define TBL_E_NONODE      (-78)  /* key="NODE" not specified in the table */

  /* [5] tb2_name() */
#define TBL_E_NONAME      (-79)  /* specified n (serial_number) is invalid */

/* typedef struct HashTable tb2_Hash; */

typedef struct dht tb2_Hash;

typedef struct _tb2_paramkey {	/* each column/parameter info */
    char *keyword;		/* keyword of each parameter */
    char type;			/* type (s/d/x/f) of parameter */
    char *format;		/* conversion format of each parameter */
} tb2_ParamKey;

typedef struct _tb2_spec {
    char *abbrev;		/* table name abbreviation */
    char *filename;		/* table file name */
    int num_param;		/* no. of columns (keywords/parameters) */
    int num_entry;		/* no. of rows (entries) */
    tb2_Hash *hash_keyword;	/* hash table pointer for keyword */
    tb2_Hash *hash_name;	/* hash table pointer for name */
    tb2_Hash *hash_serial;	/* hash table pointer for serial */
#ifdef TB2INTERNAL
    struct _tb2_paramkey	/* each column/parameter info */
	(*tb2_paramkey)[];
#else /* TB2INTERNAL */
    struct _tb2_paramkey	/* each column/parameter info */
	**tb2_paramkey;
#endif /* TB2INTERNAL */
} tb2_Spec ;

typedef union _tb2_parament {	/* each parameter info */
    char *string;		/* 's' string type */
    int integer;		/* 'd' integer type */
    long longinteger;		/* 'l' long type */
    float real;			/* 'f' real type */
    double doublereal;		/* 'g' double type */
} tb2_ParamEnt ;

typedef struct _tb2_entry {
    int serial;		/* serial number starting at 1 */
    char *name;		/* name key */
#ifdef TB2INTERNAL
    union _tb2_parament	/* each parameter info */
	(*tb2_parament)[];
#else /* TB2INTERNAL */
    union _tb2_parament	/* each parameter info */
	**tb2_parament;
#endif /* TB2INTERNAL */
} tb2_Entry ;

#ifndef __STDC__
#ifndef const
#define const			/* I should consider this again */
#endif
#endif

/* tb2_read.c */
tb2_Spec *tb2_read( char *abbrev );
int tbl_read( char *abbrev );
extern tb2_Spec *default_tbl;	/* default table for old functions */
#define tbl_errno tb2_errno
extern int tb2_errno;		/* error code */

/* _tb2_num.c */
int _tb2_num( tb2_Spec *tb2, char* name );
int _tbl_num( char *name );

/* tb2_name.c */
char *tb2_name( tb2_Spec *tb2, int serial );
char *tbl_name( int serial );

/* tb2_int.c */
int tb2_int( tb2_Spec *tb2, char *name, char *key, int *data );
int tb2_longint( tb2_Spec *tb2, char *name, char *key, long *data );
int tbl_int( char *name, char *key, int *data );
int tbl_longint( char *name, char *key, long *data );

/* tb2_get.c */
int tb2_doublereal( tb2_Spec *tb2, char* name, char *key, double *data );
int tb2_real( tb2_Spec *tb2, char* name, char *key, float *data );
int tb2_string( tb2_Spec *tb2, char* name, char *key, char **data );
int tbl_doublereal( char* name, char *key, double *data );
int tbl_real( char* name, char *key, float *data );
int tbl_string( char* name, char *key, char **data );

/* tb2_node.c */
char *tb2_node( tb2_Spec *tb2, char *name );
char *tbl_node( char *name );

/* tb2_entry.c */
tb2_ParamEnt *tb2_entry( tb2_Spec *tb2, char *name );
/* tb2_Entry *tb2_entry( tb2_Spec *tb2, char *name ); */
tb2_ParamEnt *tb2_data( tb2_Spec *tb2, char *name, char *key );
tb2_ParamEnt *tb2_sentry( tb2_Spec *tb2, int serial );
int tbl_param( char *name, int *param );

/* tb2_hash.c */
tb2_Hash *tb2_hashcreate( char typek, char typed );
int tb2_enteryhash( tb2_Spec *tb2spec, tb2_Entry *tb2entry );
#ifdef TB2INTERNAL
int tb2_keywordhash( tb2_Spec *tb2spec, struct _tb2_paramkey (*tb2paramkey)[]);
#else /* TB2INTERNAL */
int tb2_keywordhash( tb2_Spec *tb2spec, struct _tb2_paramkey **tb2paramkey);
#endif /* TB2INTERNAL */
tb2_Entry *tb2_entryfind( tb2_Spec *tb2spec, char *name, int serial, 
			  char type );
int tb2_keywordfind( tb2_Spec *tb2spec, char *keyword );

/* tb2_misc.c */
void *tb2_malloc( size_t size );
char *tb2_strdup( const char *str );

/* tb2_err.c */
void tbl_errend();
int tb2_errmsg( char *str );

#endif /* _TB2_H */
