/**
 *	tb2_entry.c	k.furukawa	july 29, 1996
 *
 * Modifications:
 *  k.f mar.11.2004  add tb2_param
 *
 *****
 *
 *  tb2_ParamEnt(*)[] tb2_entry( tb2_Spec *tb2, char *name );
 *        get a entry (array of parameters) for specified entry name
 *    tb2 -  table spec return by tb2_read
 *    name - entry name (first parameter)
 *    return: pointer to tb2_Entry structure on success
 *            0 -- error (error code in tb2_errno, see tb2.h)
 *
 *  tb2_ParamEnt * tb2_data( tb2_Spec *tb2, char *name, char *key );
 *        get a parameter (union) for specified entry name and keyword
 *    tb2 -  table spec return by tb2_read
 *    name - entry name (first parameter)
 *    key -  keyword
 *    return: pointer to tb2_ParamEnt union on success
 *            0 -- error (error code in tb2_errno, see tb2.h)
 *
 *  tb2_ParamEnt(*)[] tb2_sentry( tb2_Spec *tb2, int serial );
 *        get a entry (array of parameters) for specified serial number
 *    tb2 -  table spec returned from tb2_read
 *    name - entry name (first parameter)
 *    return: pointer to tb2_Entry structure on success
 *            0 -- error (error code in tb2_errno, see tb2.h)
 *
 *  int tbl_param( char *name, int *param );
 *        get parameters assuming all parameters are integers
 *        emulation of old interface
 *    name - entry name (first parameter)
 *    param - parameters to be returned (array of integers)
 *
 *  int tb2_param( tb2_Spec *tb2, char *name, int *param );
 *        get parameters assuming all parameters are integers
 *        emulation of old interface
 *    tb2 -  table spec return by tb2_read
 *    name - entry name (first parameter)
 *    param - parameters to be returned (array of integers)
**/
static char what_tb2entry[] =
"@(#)tb2_entry.c, v2.1.1, k.f, linac table get entry, jul.1996-mar.2004";

/*** definitions ***/
#include  "tb2.h"		/* error codes, default directory */

/**
 *   tb2_entry() -- get specified entry from table
**/
tb2_ParamEnt * tb2_entry( tb2_Spec *tb2, char * name )
/* tb2_ParamEnt (*)[] tb2_entry was syntax error */
/* tb2_Spec *tb2;		table specification */
/* char *name;			entry name (first parameter string) */
{
    tb2_Entry *entry;			/* table entry */

    if ( !(tb2) )  goto EUNREAD;

    if ( !(entry = tb2_entryfind( tb2, name, 0, 'n' )) )  goto ENAME;

    return ( (tb2_ParamEnt *) (entry->tb2_parament) );

 EUNREAD:
#ifdef TBL_WARN
    fprintf( WARN_DEST, "tb2_entry %s: table is not yet read\n", TBL_VER );
#endif
    tb2_errno = TBL_E_UNREAD;
    return (void *) NULL;

 ENAME:
#ifdef TBL_WARN
    fprintf( WARN_DEST, "tb2_entry %s: name '%s' not found in table '%s'\n", 
	    TBL_VER, name, tb2->filename );
#endif
    tb2_errno = TBL_E_NAME;
    return (void *) NULL;
}

/**
 *   tb2_data() -- get specified parameter data of entry from table
**/
tb2_ParamEnt * tb2_data( tb2_Spec *tb2, char *name, char *key )
{
    tb2_ParamEnt(*aparament)[];		/* array of parameters in an entry */
    int index;				/* keyword index */

    if ( !(tb2) )  goto EUNREAD;

    if ( !(aparament = (tb2_ParamEnt(*)[]) tb2_entry( tb2, name )) )
	goto ENAME;

    if ( -1 == (index = tb2_keywordfind( tb2, key )) )  goto EKEY;

    return &((*aparament)[index]);

 EUNREAD:
#ifdef TBL_WARN
    fprintf( WARN_DEST, "tb2_data %s: table is not read yet\n", TBL_VER );
#endif
    tb2_errno = TBL_E_UNREAD;
    return (void *) NULL;

 ENAME:
    tb2_errno = TBL_E_NAME;
    return (void *) NULL;

 EKEY:
#ifdef TBL_WARN
    fprintf( WARN_DEST, "tb2_data %s: key '%s' not found in table '%s'\n", 
	    TBL_VER, key, tb2->filename );
#endif
    tb2_errno = TBL_E_KEY;
    return (void *) NULL;
}

/**
 *   tb2_sentry() -- get specified entry from table
**/
tb2_ParamEnt * tb2_sentry( tb2_Spec *tb2, int serial )
/* tb2_Spec *tb2;		table specification */
/* int serial;			serial number */
{
    tb2_Entry *entry;			/* table entry */

    if ( !(tb2) )  goto EUNREAD;

    if ( !(entry = tb2_entryfind( tb2, (void *)NULL, serial, 's' )) )
	goto ENAME;

    return ( (tb2_ParamEnt *) (entry->tb2_parament) );

 EUNREAD:
#ifdef TBL_WARN
    fprintf( WARN_DEST, "tb2_sentry %s: table is not read yet\n", TBL_VER );
#endif
    tb2_errno = TBL_E_UNREAD;
    return (void *) NULL;

 ENAME:
#ifdef TBL_WARN
    /* bug-fix by n.k. feb.18.1999 */
    fprintf( WARN_DEST, "tb2_sentry %s: %d-th name not found in table '%s'\n",  
            TBL_VER, serial, tb2->filename );
#if 0
    fprintf( WARN_DEST, "tb2_sentry %s: name '%s' not found in table '%s'\n", 
	    TBL_VER, name, tb2->filename );
#endif
#endif
    tb2_errno = TBL_E_NAME;
    return (void *) NULL;
}

/*  int tbl_param( char *name, int *param );
 *        get parameters assuming all parameters are integers
 *        emulation of old interface
 *    name - entry name (first parameter)
 *    param - parameters to be returned (array of integers)
 */
tbl_param( char *name, int *param )
{
    tb2_Entry *entry;			/* table entry structure */
    int offset;				/* offset in parameters */
    int i;

    if ( !(default_tbl) )  goto EUNREAD;

    if ( !(entry = tb2_entryfind( default_tbl, name, 0, 'n' )) )  goto ENAME;

    if ( !strcmp( (*(default_tbl->tb2_paramkey))[1].keyword, "NODE" ) )
	offset = 2;
    else  offset = 1;

    for ( i=0; i<ORIG_N_KEYS-offset; i++ )
	*(param+i) = (*(entry->tb2_parament))[i+offset].integer;

    return 0;

 EUNREAD:
#ifdef TBL_WARN
    fprintf( WARN_DEST, "tbl_param %s: table is not read yet\n", TBL_VER );
#endif
    tb2_errno = TBL_E_UNREAD;
    return -1;

 ENAME:
#ifdef TBL_WARN
    /* bug-fix by n.k. feb.18.1999 */
    fprintf( WARN_DEST, "tb2_param %s: name '%s' not found\n", 
            TBL_VER, name );
#if 0
    fprintf( WARN_DEST, "tb2_param %s: name '%s' not found in table '%s'\n", 
            TBL_VER, name, tb2->filename );
#endif
#endif
    tb2_errno = TBL_E_NAME;
    return -1;
}

/*  int tb2_param( tb2_Spec *tb2, char *name, int *param );
 *        get parameters assuming all parameters are integers
 *        emulation of old interface
 *    tb2 -  table spec return by tb2_read
 *    name - entry name (first parameter)
 *    param - parameters to be returned (array of integers)
 */
tb2_param( tb2_Spec *tb2, char *name, int *param )
{
    tb2_Entry *entry;			/* table entry structure */
    int offset;				/* offset in parameters */
    int i;

    if ( !(tb2) )  goto EUNREAD;

    if ( !(entry = tb2_entryfind( tb2, name, 0, 'n' )) )  goto ENAME;

    if ( !strcmp( (*(tb2->tb2_paramkey))[1].keyword, "NODE" ) )
	offset = 2;
    else  offset = 1;

    for ( i=0; i<ORIG_N_KEYS-offset; i++ )
	*(param+i) = (*(entry->tb2_parament))[i+offset].integer;

    return 0;

 EUNREAD:
#ifdef TBL_WARN
    fprintf( WARN_DEST, "tb2_param %s: table is not read yet\n", TBL_VER );
#endif
    tb2_errno = TBL_E_UNREAD;
    return -1;

 ENAME:
#ifdef TBL_WARN
    /* bug-fix by n.k. feb.18.1999 */
    fprintf( WARN_DEST, "tb2_param %s: name '%s' not found\n", 
            TBL_VER, name );
#if 0
    fprintf( WARN_DEST, "tb2_param %s: name '%s' not found in table '%s'\n", 
            TBL_VER, name, tb2->filename );
#endif
#endif
    tb2_errno = TBL_E_NAME;
    return -1;
}
