/**
 *	tb2_int.c	k.f.	july 29, 1996
 *
 *  int tb2_doublereal( tb2_Spec *tb2, char* name, char *key, double *data )  
 *         --  get doublereal parameter (data) for name by specifing keyword
 *    tb2  - table spec returned from tb2_read()
 *    name - name to be searched (in the first row, KEY="NAME")
 *    key  - parameter keyword to be searched, defined in the table itself
 *    data - double data for specified name and keyword
 *  return: n=0  -- successfully completed
 *          < 0  -- error (see tb2.h)
 *
 *  int tb2_real( tb2_Spec *tb2, char* name, char *key, float *data )  
 *         --  get (single) real parameter (data) for name by specifing keyword
 *    tb2  - table spec returned from tb2_read()
 *    name - name to be searched (in the first row, KEY="NAME")
 *    key  - parameter keyword to be searched, defined in the table itself
 *    data - float data for specified name and keyword
 *  return: n=0  -- successfully completed
 *          < 0  -- error (see tb2.h)
 *
 *  int tb2_string( tb2_Spec *tb2, char* name, char *key, char **data )  
 *         --  get string parameter (data) for name by specifing keyword
 *    tb2  - table spec returned from tb2_read()
 *    name - name to be searched (in the first row, KEY="NAME")
 *    key  - parameter keyword to be searched, defined in the table itself
 *    data - string data for specified name and keyword
 *  return: n=0  -- successfully completed
 *          < 0  -- error (see tb2.h)
 *
 *  int tbl_doublereal( char* name, char *key, double *data )
 *         --  get double parameter (data) for name by specifing keyword
 *
 *  int tbl_real( char* name, char *key, float *data )
 *         --  get float parameter (data) for name by specifing keyword
 *
 *  int tbl_string( char* name, char *key, char **data )
 *         --  get string parameter (data) for name by specifing keyword
 *
 * Modifications:
**/
static char what_tb2get[] =
"@(#)tb2_get.c, v2.1.2, k.f, linac table get-real-string, jul.1996-jun.1997";

/*** definitions ***/
#include  "tb2.h"         /* error codes, default directory */

/**
 *   tb2_doublereal() - get real parameter for name and keyword
 *	tb2  - table
 *	name - entry name
 *	key  - keyword
 *	data - parameter to be returned
 *	return: 0  success
 *		<0 failure
**/
int tb2_doublereal( tb2, name, key, data )
tb2_Spec *tb2;			/* table specification */
char  *name;			/* name to be searched */
char  *key;			/* parameter keyword defined in the table */
double   *data;			/* double parameter */
{
    tb2_ParamEnt * parament;		/* parameter */

    if ( !(parament = tb2_data( tb2, name, key )) )  {
	return tb2_errno;
    }
    *data = parament->doublereal;
    return 0;
}

/**
 *   tb2_real() - get (single) real parameter for name and keyword
 *	tb2  - table
 *	name - entry name
 *	key  - keyword
 *	data - parameter to be returned
 *	return: 0  success
 *		<0 failure
**/
int tb2_real( tb2, name, key, data )
tb2_Spec *tb2;			/* table specification */
char  *name;			/* name to be searched */
char  *key;			/* parameter keyword defined in the table */
float  *data;			/* float parameter */
{
    tb2_ParamEnt * parament;		/* parameter */

    if ( !(parament = tb2_data( tb2, name, key )) )  {
	return tb2_errno;
    }
    *data = parament->real;
    return 0;
}

/**
 *   tb2_string() - get string parameter for name and keyword
 *	tb2  - table
 *	name - entry name
 *	key  - keyword
 *	data - parameter to be returned
 *	return: 0  success
 *		<0 failure
**/
int tb2_string( tb2, name, key, data )
tb2_Spec *tb2;			/* table specification */
char  *name;			/* name to be searched */
char  *key;			/* parameter keyword defined in the table */
char  **data;			/* string parameter */
{
    tb2_ParamEnt * parament;		/* parameter */

    if ( !(parament = tb2_data( tb2, name, key )) )  {
	return tb2_errno;
    }
    *data = parament->string;
    return 0;
}

/**
 *   tbl_doublereal() - get double real parameter for name and keyword
**/
int tbl_doublereal( name, key, data )
char  *name;			/* name to be searched */
char  *key;			/* parameter keyword defined in the table */
double   *data;			/* double real parameter */
{
    return tb2_doublereal( default_tbl, name, key, data );
}

/**
 *   tbl_real() - get (single) real parameter for name and keyword
**/
int tbl_real( name, key, data )
char  *name;			/* name to be searched */
char  *key;			/* parameter keyword defined in the table */
float  *data;			/* real parameter */
{
    return tb2_real( default_tbl, name, key, data );
}
/**
 *   tbl_string() - get string parameter for name and keyword
**/
int tbl_string( name, key, data )
char  *name;			/* name to be searched */
char  *key;			/* parameter keyword defined in the table */
char  **data;			/* string parameter */
{
    return tb2_string( default_tbl, name, key, data );
}
