/**
 *	tbl_int.c	n.k.	june 9, 1992
 *	tb2_int.c	k.f.	july 29, 1996
 *
 *  int tb2_int( tb2_Spec tb2, char* name, char *key, int *data )  
 *         --  get integer parameter (data) for name by specifing keyword
 *    tb2  - table spec returned from tb2_read()
 *    name - name to be searched (in the first row, KEY="NAME")
 *    key  - parameter keyword to be searched, defined in the table itself
 *    data - integer data for specified name and keyword
 *  return: n=0  -- successfully completed
 *          < 0  -- error (see tb2.h)
 *
 *  int tb2_longint( tb2_Spec tb2, char* name, char *key, long *data )  
 *         --  get long integer parameter (data) for name by specifing keyword
 *    tb2  - table spec returned from tb2_read()
 *    name - name to be searched (in the first row, KEY="NAME")
 *    key  - parameter keyword to be searched, defined in the table itself
 *    data - long integer data for specified name and keyword
 *  return: n=0  -- successfully completed
 *          < 0  -- error (see tb2.h)
 *
 *  int tbl_int( char* name, char *key, int *data )     (old interface)
 *         --  get integer parameter (data) for name by specifing keyword
 *
 *  int tbl_longint( char* name, char *key, long *data )     (old interface)
 *         --  get long integer parameter (data) for name by specifing keyword
 *
 * Modifications:
**/
static char what_tb2int[] =
"@(#)tb2_int.c, v2.1.1, n.k, k.f, linac table get-int, jun.1992-jun.1997";

/*** definitions ***/
#include  "tb2.h"         /* error codes, default directory */

/**
 *   tb2_int() - get integer parameter for name and keyword
 *	tb2  - table
 *	name - entry name
 *	key  - keyword
 *	data - parameter to be returned
 *	return: 0  success
 *		<0 failure
**/
int tb2_int( tb2, name, key, data )
tb2_Spec *tb2;			/* table specification */
char  *name;			/* name to be searched */
char  *key;			/* parameter keyword defined in the table */
int   *data;			/* integer parameter */
{
    tb2_ParamEnt * parament;		/* parameter */

    if ( !(parament = tb2_data( tb2, name, key )) )  {
	return tb2_errno;
    }
    *data = parament->integer;
    return 0;
}

/**
 *   tb2_longint() - get long integer parameter for name and keyword
 *	tb2  - table
 *	name - entry name
 *	key  - keyword
 *	data - parameter to be returned
 *	return: 0  success
 *		<0 failure
**/
int tb2_longint( tb2, name, key, data )
tb2_Spec *tb2;			/* table specification */
char  *name;			/* name to be searched */
char  *key;			/* parameter keyword defined in the table */
long   *data;			/* long integer parameter */
{
    tb2_ParamEnt * parament;		/* parameter */

    if ( !(parament = tb2_data( tb2, name, key )) )  {
	return tb2_errno;
    }
    *data = parament->longinteger;
    return 0;
}

/**
 *   tbl_int() - get integer parameter for name and keyword (old interface)
**/
int tbl_int( name, key, data )
char  *name;			/* name to be searched */
char  *key;			/* parameter keyword defined in the table */
int   *data;			/* integer parameter */
{
    return tb2_int( default_tbl, name, key, data );
}

/**
 *   tbl_longint() - get long integer parameter for name and keyword (old interface)
**/
int tbl_longint( name, key, data )
char  *name;			/* name to be searched */
char  *key;			/* parameter keyword defined in the table */
long   *data;			/* long integer parameter */
{
    return tb2_longint( default_tbl, name, key, data );
}
