/**
 *	tb2_regex.c	k.furukawa	jun 12 1997.
 *
 *	regular expression handling for linac table routines

 *	
 * k.furukawa, feb.14.1997.  replace dynhash-1.0 with dht-1.2
 *
 * tb2_Hash * tb2_hashcreate( char typek, char typed );
 *     create hash table with specified key/data type
 *
 * int tb2_enteryhash( tb2_Spec *tb2spec, tb2_Entry *tb2entry );
 *     enter a entry into double hash table
 *
 * int tb2_keywordhash( tb2_Spec *tb2spec, struct _tb2_paramkey (*tb2key)[]);
 *     enter a keyword into hash table
 *
 * tb2_Entry * tb2_entryfind( tb2_Spec *tb2spec, char *name, int serial, 
 *				char type );
 *     find an entry for specified name/serial (row)
 *
 * int tb2_keywordfind( tb2_Spec *tb2spec, char *keyword );
 *     find an index number (start at 0) for the specified keyword (column)
 **/

static char what_tb2regex[] =
"@(#)tb2_regex.c, v2.1.0, k.furukawa, linac table regex, jun.1997";

#include <tb2.h>

/* tb2_hashcreate()  --  interface to dynamic hash library */
/* create hash table from key to data */
/* arguments are data type, 's':string, 'i':integer, 'm':structure */
tb2_Hash * tb2_hashcreate( char typek, char typed )
{
    tb2_Hash * ht;			/* hash table data structure */
    dhtValueType keytyp;		/* hash key type */
    dhtValueType datatyp;		/* data type */

    switch ( typek ) {
    case 's':	keytyp = dhtStringValue; break;
    case 'i':	keytyp = dhtSimpleValue; break;
    case 'c':	keytyp = dhtCompactMemoryValue; break;
    case 'm':	keytyp = dhtMemoryValue; break;
    default:
#ifdef DEBUG1
	fprintf( stderr, "tb2_hashcreate illegal type specified\n" );
#endif /* DEBUG1 */
	return (void *)NULL;
    }

    switch ( typed ) {
    case 's':	datatyp = dhtStringValue; break;
    case 'i':	datatyp = dhtSimpleValue; break;
    case 'c':	datatyp = dhtCompactMemoryValue; break;
    case 'm':	datatyp = dhtMemoryValue; break;
    default:
#ifdef DEBUG1
	fprintf( stderr, "tb2_hashcreate illegal type specified\n" );
#endif /* DEBUG1 */
	return (void *)NULL;
    }

    /* CreateHashTable(ValueType KeyTyp, Boolean CopyKey, */
    /*                 ValueType DataTyp, Boolean CopyData); */
    /* if ((ht = CreateHashTable(keytyp, False, MemoryValue, True)) */
#ifdef HASHTABLENOCOPYDATA
    if ((ht = CreateHashTable(keytyp, dhtNoCopy, datatyp, dhtNoCopy))
	== NilHashTable)  {
#else /* HASHTABLENOCOPYDATA */
#ifdef HASHTABLENOSIMPLE
    if ((ht = CreateHashTable(keytyp, dhtNoCopy, datatyp, dhtCopy))
	== NilHashTable)  {
#else /* HASHTABLENOSIMPLE */
    if ( datatyp == dhtMemoryValue )  datatyp = dhtSimpleValue;

    if ((ht = CreateHashTable(keytyp, dhtNoCopy, datatyp, dhtNoCopy))
	== NilHashTable)  {
#endif /* HASHTABLENOSIMPLE */
#endif /* HASHTABLENOCOPYDATA */

#ifdef DEBUG1
	fprintf( stderr, "tb2_hashcreate failure, no memory\n" );
#endif /* DEBUG1 */
	return (void *)NULL;
    }
    return ht;
}

/* tb2_enterhash()  --  interface to dynamic hash library */
/* enter a entry into hash table */
int tb2_enterhash( tb2_Spec *tb2spec, tb2_Entry *tb2entry )
{
    dhtValue Key, Data;
    MemVal mv;
    
    if ( !tb2spec )  return -1;

#ifdef HASHTABLENOSIMPLE
    mv.Leng = sizeof (tb2_Entry);
    mv.Data = (void *)tb2entry;
    /* Data.Mem = &mv; */
    Data = (dhtValue) &mv;
#else /* HASHTABLENOSIMPLE */
    Data = (dhtValue) tb2entry;
#endif /* HASHTABLENOSIMPLE */

    /* Key.Str = tb2entry->name; */				/* name key */
    Key = tb2entry->name;				/* name key */
    if (NilHashElement == 				/* enter name hash */
	EnterHashElement( tb2spec->hash_name, Key, Data ))  {
#ifdef DEBUG1
	fprintf( stderr, "tb2_entryhash failure, no memory\n" );
#endif /* DEBUG1 */
	return -1;
    }
#ifdef DEBUG1
    fprintf( stderr, 
     "tb2_entryhash Key.Str:'%s' Data.Mem->Leng:%d. Data.Mem->Data:%p\n",
	     /* Key.Str, Data.Mem->Leng, Data.Mem->Data); */
	     Key, Data->Leng, Data->Data);
#endif /* DEBUG1 */

    /* Key.Smpl = tb2entry->serial; */			/* serial key */
    Key = (dhtValue)(tb2entry->serial);			/* serial key */
    if (NilHashElement == 				/* enter serial hash */
	EnterHashElement( tb2spec->hash_serial, Key, Data ))  {
#ifdef DEBUG1
	fprintf( stderr, "tb2_entryhash failure, no memory\n" );
#endif /* DEBUG1 */
	return -1;
    }
#ifdef DEBUG1
    fprintf( stderr, 
     "tb2_entryhash Key.Smpl:%d. Data.Mem->Leng:%d. Data.Mem->Data:%p\n",
	     /* Key.Smpl, Data.Mem->Leng, Data.Mem->Data); */
	     (int)Key, Data->Leng, Data->Data);
#endif /* DEBUG1 */

    return 0;
}

/* tb2_keywordhash()  --  interface to dynamic hash library */
/* enter a keywords into hash table */
int tb2_keywordhash( tb2_Spec *tb2spec, struct _tb2_paramkey (*tb2paramkey)[] )
{
    dhtValue Key, Data;
    char *keyword;
    int i = 0;

    if ( !tb2spec )  return -1;

    while ( keyword = (*tb2paramkey)[i].keyword ) {	/* for all keywords */
	/* Key.Str = keyword; */			/* string key */
	Key = keyword;					/* string key */
	/* Data.Smpl = i; */				/* simple value data */
	Data = (dhtValue) i;				/* simple value data */

	if (NilHashElement ==				/* enter */
	    EnterHashElement( tb2spec->hash_keyword, Key, Data ))  {
#ifdef DEBUG1
	    fprintf( stderr, "tb2_keywordhash failure, no memory\n" );
#endif /* DEBUG1 */
	    return -1;					/* no memory */
	}

	i++;						/* next */
    }
    return 0;
}

/* tb2_entryfind()  --  interface to dynamic hash library */
/* find entry in hash table and return entry structure */
/* type argument is key type, 'n':name, 's':serial */
tb2_Entry * tb2_entryfind( tb2_Spec *tb2spec, char *name, int serial, 
			  char type )
{
    dhtElement *elem;			/* hash element, pointer to data */
    tb2_Hash *hash;			/* hash table to be used */
    dhtValue key;			/* key to be used */

    /* if ( !tb2spec )  return (void *)NULL; */
    if ( !tb2spec )  {
	tb2_errno = TBL_E_UNREAD;
	return (void *)NULL;
    }

    switch ( type )  {
    case 'n':					/* name key specified */
	/* key.Str = name; */			/* string value key */
	key = name;				/* string value key */
	hash = tb2spec->hash_name;		/* name hash */
	break;

    case 's':					/* serial key specified */
	/* key.Smpl = serial; */		/* simple value key */
	key = (dhtValue) serial;		/* simple value key */
	hash = tb2spec->hash_serial;		/* serial number hash */
	break;

    default:
#ifdef DEBUG1
	fprintf( stderr, "tb2_entryfind illegal type specified\n" );
#endif /* DEBUG1 */
	tb2_errno = TBL_E_KEYWORDS;
	return (void *)NULL;			/* error */
    }
	
    if ( NilHashElement == 			/* find */
	(elem = LookupHashElement( hash, key )) )  {
#ifdef DEBUG1
	fprintf( stderr, "tb2_entryfind failure, not found\n" );
#endif /* DEBUG1 */
#ifdef DEBUG2
	fDumpHashTable(0, hash, stderr);
#endif /* DEBUG2 */
	tb2_errno = TBL_E_NAME;
	return (void *)NULL;			/* not found */
    }
#ifdef DEBUG1
    switch ( type ) {
    case 'n':
      fprintf( stderr, 
     "tb2_entryfind Key.Str:'%s' Data.Mem->Leng:%d. Data.Mem->Data:%p\n",
	     /* key.Str, elem->Data.Mem->Leng, elem->Data.Mem->Data); */
	     key, ((MemVal*)(elem->Data))->Leng,((MemVal*)(elem->Data))->Data);
      break;
    case 's':
      fprintf( stderr, 
     "tb2_entryfind Key.Smpl:%d. Data.Mem->Leng:%d. Data.Mem->Data:%p\n",
	     /* key.Smpl, elem->Data.Mem->Leng, elem->Data.Mem->Data); */
	     key, ((MemVal*)(elem->Data))->Leng,((MemVal*)(elem->Data))->Data);
      break;
    }
#endif /* DEBUG1 */

#ifdef HASHTABLENOSIMPLE
    return (tb2_Entry *)(((MemVal*)(elem->Data))->Data); /* entry found */
#else /* HASHTABLENOSIMPLE */
    return (tb2_Entry *)(elem->Data); 			/* entry found */
#endif /* HASHTABLENOSIMPLE */
}

/* tb2_keywordfind()  --  interface to dynamic hash library */
/* find keyword in keyword hash table and return index (start at 0) */
int tb2_keywordfind( tb2_Spec *tb2spec, char *keyword )
{
    dhtElement *elem;			/* hash element, pointer to data */
    tb2_Hash *hash;			/* hash table to be used */
    dhtValue key;			/* key to be used */

    /* if ( !tb2spec )  return -1; */
    if ( !tb2spec )  {
	tb2_errno = TBL_E_UNREAD;
	return -1;
    }

    /* key.Str = keyword; */			/* string value key */
    key = keyword;				/* string value key */
    hash = tb2spec->hash_keyword;		/* name hash */
	
    if ( NilHashElement == 			/* find */
	(elem = LookupHashElement( hash, key )) )  {
#ifdef DEBUG1
	fprintf( stderr, "tb2_keywordfind failure, not found\n" );
#endif /* DEBUG1 */
	tb2_errno = TBL_E_KEY;
	return -1;				/* not found */
    }
    /* return (int)(elem->Data.Smpl); */	/* entry found */
    return (int)(elem->Data);			/* entry found */
}
