/**
 *  tb2key  - get paramater for name & keyword
 *    original version n.k. june 1992
 *  rewritten for v2.0 tbllib
 *    k.furukawa, oct.1.1996.
 *
 * Restrictions:
 *  formats in table file should be used in printf() as well as scanf()
 *  if it contains illegal formats, results are undefined
 *
 * Note:
 *  cc -std1 -O -DTB2INTERNAL -I. -I/usr/users/control/include -o tb2key tb2key.c ~control/lib/libtb2.a
 *
 * Modifications:
 *  nov.11.1999 n.k.  allow multiple keys  (v2.1.1 <- v2.0.2)
 *  sep.13.2000 n/k/  alow stdin input (v2.1.2)
**/
static char what_tb2key[] =
"@(#)tb2key.c, v2.1.2, n.f, k.f, linac table search, jun.1992-sep.2000";

/*** definitions ***/
#include  <stdio.h>
#include  "tb2.h"


/**************************************************************************
 *   main of tb2key
***************************************************************************/
main( argc, argv )
    int    argc;
    char   *argv[];
{
    tb2_Spec *table;
    tb2_ParamEnt * parament;
    int keyindex;
    char format[L_FORMAT];
    char oneword[128], oneline[256], onename[256];
    int  i,j,k;

/*** [0] initialize ***/

    if( argc < 4 )  goto USAGE;

    if ( !(table = tb2_read( argv[1] )) )  {
	fprintf( stderr, "tb2_read: table open failure, code = %d\n", 
		tb2_errno );
	tbl_errend();
    }


/*** [1] name in - from argv[2] or from stdin ***/

    /* argv[2] = name, or '-' */

    while( 1 ) {

      /* determine onename */
      if( argv[2][0] == '-' ) {      /* read from stdin ? */
      AGAIN:
	if( fgets( onename, 256, stdin ) != NULL ) {
	  i = strlen(onename); if( i<1 ) i=1; /* remove CR */
	  onename[i-1] = '\0';
	  if( onename[0]=='!' || onename[0]==';' || onename[0]=='#' ) goto AGAIN;
	} else {
	  break;               /* EOF - exit while */
	}
      } else {
	strcpy( onename, argv[2] );  /* use specified name */
      }
      for( i=0; i<strlen(onename) ; i++ ) { /* remove TAB */
	if( onename[i] == '\t' )  onename[i]='\0'; 
      }

  /*** [2] keyword in - from argv[i+3] ***/

    /* [2-1] clear output buffer */
    oneline[0] = '\0';           /* clear output buffer */

    /* [2-2] start of for - keyword in */
    for( i=0; i<(argc-3) ; i++ )  {
      if ( -1 == (keyindex = tb2_keywordfind( table, argv[i+3] )) )  {
	fprintf( stderr, "tb2_keyword: keyword '%s' find failure, code = %d\n", 
		argv[i+3], tb2_errno );
	tbl_errend();
      }
      
      if ( !(parament = tb2_data( table, onename, argv[i+3] )) )  {
	fprintf( stderr, "tb2_data: name '%s' find failure, code = %d\n",
		onename, tb2_errno );
	tbl_errend();
      }

      strcpy( format, ((*(table->tb2_paramkey))[keyindex]).format );
      format[ strlen(format) -2 ] = '\0';		/* remove last '%n' */
      
      switch (((*(table->tb2_paramkey))[keyindex]).type)  {
      case 's':
	sprintf( oneword, format, parament->string );
	break;
      case 'd':
	sprintf( oneword, format, parament->integer );
	break;
      case 'l':
	sprintf( oneword, format, parament->longinteger );
	break;
      case 'f':
	sprintf( oneword, format, parament->real );
	break;
      case 'g':
	sprintf( oneword, format, parament->doublereal );
	break;
      }
      if( i != 0 )  strcat( oneline, " " );
      strcat( oneline, oneword );

      /* [2-3] end of for - end of keyword in  */
    }

    /* [2-4] output result (one line) */
    fprintf( stdout, "%s\n", oneline  );  /* output as one line */
    fflush( stdout );

    /* [2-5] exit from while ? */
    if( argv[2][0] != '-' )  break;       /* single name specified */


/*** [1] end of name in - end of while ***/
    }


/*** [E] end of program ***/

    exit( 0 );


/*** [Info] Usage printout ***/
USAGE:
  fprintf( stderr, "%s\n\n", what_tb2key );
  fprintf( stderr, "%s %s - search table with keyword\n\n", argv[0], TBL_VER );
  fprintf( stderr, "Usage: %s abbr name key\n", argv[0] );
  fprintf( stderr, "       abbr -- abbreviation of table filename\n" );
  fprintf( stderr, "       name -- name to be searched\n" );
  fprintf( stderr, "       key  -- keyword for name to be searched\n" );
  fprintf( stderr, "Usage2: %s abbr name key1 key2 key3 key4\n", argv[0] );
  fprintf( stderr, "        (... multiple keywords)\n" );
  fprintf( stderr, "Usage3: %s abbr - key1 key2 key3 key4 <namelist.lst\n", argv[0] );
  fprintf( stderr, "        (... multiple names from stdin)\n" );
  fprintf( stderr, " \n" );
  fprintf( stderr, "Example: %s mg BM_A1_C1/2/3/4 INTERLOCK\n", argv[0]);
  fprintf( stderr, "         %s tbl TAMA NODE\n", argv[0]);
  fprintf( stderr, "         %s mg BM_61_1/6 PARAM\n", argv[0]);
  fprintf( stderr, "         %s sp SP_A4_2 NODE SECT CH\n", argv[0]);
  exit( 1 );
}
